"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const typedoc_1 = require("typedoc");
const events_1 = require("typedoc/dist/lib/output/events");
const theme_1 = require("typedoc/dist/lib/output/theme");
const breadcrumbs_component_1 = require("./components/breadcrumbs.component");
const comments_component_1 = require("./components/comments.component");
const options_component_1 = require("./components/options.component");
class MarkdownTheme extends theme_1.Theme {
    constructor(renderer, basePath) {
        super(renderer, basePath);
        this.indexName = 'README';
        this.fileExt = '.md';
        this.listenTo(renderer, events_1.PageEvent.END, this.onPageEnd, 1024);
        renderer.removeComponent('assets');
        renderer.removeComponent('javascript-index');
        renderer.removeComponent('toc');
        renderer.removeComponent('pretty-print');
        renderer.addComponent('breadcrumbs', new breadcrumbs_component_1.BreadcrumbsComponent(renderer));
        renderer.addComponent('comments', new comments_component_1.CommentsComponent(renderer));
        renderer.addComponent('options', new options_component_1.OptionsComponent(renderer));
    }
    isOutputDirectory(outputDirectory) {
        let isOutputDirectory = true;
        const listings = fs.readdirSync(outputDirectory);
        if (!listings.includes(this.indexName + this.fileExt)) {
            isOutputDirectory = false;
            return;
        }
        listings.forEach(listing => {
            if (!this.allowedDirectoryListings().includes(listing)) {
                isOutputDirectory = false;
                return;
            }
        });
        return isOutputDirectory;
    }
    allowedDirectoryListings() {
        return [
            this.indexName + this.fileExt,
            'globals' + this.fileExt,
            'classes',
            'enums',
            'interfaces',
            'modules',
            'media',
            '.DS_Store',
        ];
    }
    getUrls(project) {
        const urls = [];
        const entryPoint = this.getEntryPoint(project);
        if (this.application.options.getValue('readme') === 'none') {
            entryPoint.url = this.indexName + this.fileExt;
            urls.push(new typedoc_1.UrlMapping(this.indexName + this.fileExt, entryPoint, 'reflection.hbs'));
        }
        else {
            entryPoint.url = 'globals' + this.fileExt;
            urls.push(new typedoc_1.UrlMapping('globals' + this.fileExt, entryPoint, 'reflection.hbs'));
            urls.push(new typedoc_1.UrlMapping(this.indexName + this.fileExt, project, 'index.hbs'));
        }
        if (entryPoint.children) {
            entryPoint.children.forEach((child) => {
                if (child instanceof typedoc_1.DeclarationReflection) {
                    this.buildUrls(child, urls);
                }
            });
        }
        return urls;
    }
    buildUrls(reflection, urls) {
        const mapping = typedoc_1.DefaultTheme.getMapping(reflection);
        if (mapping) {
            if (!reflection.url || !typedoc_1.DefaultTheme.URL_PREFIX.test(reflection.url)) {
                const url = mapping.directory + '/' + typedoc_1.DefaultTheme.getUrl(reflection) + this.fileExt;
                urls.push(new typedoc_1.UrlMapping(url, reflection, mapping.template));
                reflection.url = url;
                reflection.hasOwnDocument = true;
            }
            for (const child of reflection.children || []) {
                if (mapping.isLeaf) {
                    this.applyAnchorUrl(child, reflection);
                }
                else {
                    this.buildUrls(child, urls);
                }
            }
        }
        else if (reflection.parent) {
            this.applyAnchorUrl(reflection, reflection.parent);
        }
        return urls;
    }
    applyAnchorUrl(reflection, container) {
        if (!reflection.url || !typedoc_1.DefaultTheme.URL_PREFIX.test(reflection.url)) {
            const anchor = this.toAnchorRef(reflection);
            reflection.url = container.url + '#' + anchor;
            reflection.anchor = anchor;
            reflection.hasOwnDocument = false;
        }
        reflection.traverse(child => {
            if (child instanceof typedoc_1.DeclarationReflection) {
                this.applyAnchorUrl(child, container);
            }
        });
    }
    toAnchorRef(reflection) {
        function parseAnchorRef(ref) {
            return ref.replace(/"/g, '').replace(/ /g, '-');
        }
        let anchorPrefix = '';
        reflection.flags.forEach(flag => (anchorPrefix += `${flag}-`));
        const prefixRef = parseAnchorRef(anchorPrefix);
        const reflectionRef = parseAnchorRef(reflection.name);
        const anchorRef = prefixRef + reflectionRef;
        return anchorRef.toLowerCase();
    }
    getEntryPoint(project) {
        const entryPoint = this.owner.entryPoint;
        if (entryPoint) {
            const reflection = project.getChildByName(entryPoint);
            if (reflection) {
                if (reflection instanceof typedoc_1.ContainerReflection) {
                    return reflection;
                }
                else {
                    this.application.logger.warn('The given entry point `%s` is not a container.', entryPoint);
                }
            }
            else {
                this.application.logger.warn('The entry point `%s` could not be found.', entryPoint);
            }
        }
        return project;
    }
    getNavigation(project) {
        function createNavigationGroup(name, url = null) {
            const navigationGroup = new typedoc_1.NavigationItem(name, url);
            navigationGroup.children = [];
            delete navigationGroup.cssClasses;
            delete navigationGroup.reflection;
            return navigationGroup;
        }
        function getNavigationGroup(reflection) {
            if (reflection.kind === typedoc_1.ReflectionKind.ExternalModule) {
                return externalModulesNavigation;
            }
            if (reflection.kind === typedoc_1.ReflectionKind.Module) {
                return modulesNavigation;
            }
            if (reflection.kind === typedoc_1.ReflectionKind.Class) {
                return classesNavigation;
            }
            if (reflection.kind === typedoc_1.ReflectionKind.Enum) {
                return enumsNavigation;
            }
            if (reflection.kind === typedoc_1.ReflectionKind.Interface) {
                return interfacesNavigation;
            }
            return null;
        }
        function addNavigationItem(reflection, parentNavigationItem, group) {
            let navigationGroup;
            if (group) {
                navigationGroup = group;
            }
            else {
                navigationGroup = getNavigationGroup(reflection);
            }
            let titlePrefix = '';
            if (parentNavigationItem && parentNavigationItem.title) {
                titlePrefix = parentNavigationItem.title.replace(/\"/g, '') + '.';
            }
            const title = titlePrefix + reflection.name.replace(/\"/g, '');
            const url = reflection.url;
            const nav = new typedoc_1.NavigationItem(title, url, parentNavigationItem);
            nav.parent = parentNavigationItem;
            navigationGroup.children.push(nav);
            if (reflection.children) {
                reflection.children.forEach(reflectionChild => {
                    if (reflectionChild.hasOwnDocument) {
                        addNavigationItem(reflectionChild, nav, navigationGroup);
                    }
                });
            }
            delete nav.cssClasses;
            delete nav.reflection;
            return nav;
        }
        const isModules = this.application.options.getValue('mode') === 1;
        const navigation = createNavigationGroup(project.name, this.indexName + this.fileExt);
        const externalModulesNavigation = createNavigationGroup('External Modules');
        const modulesNavigation = createNavigationGroup('Modules');
        const classesNavigation = createNavigationGroup('Classes');
        const enumsNavigation = createNavigationGroup('Enums');
        const interfacesNavigation = createNavigationGroup('Interfaces');
        if (!isModules) {
            project.groups.forEach(group => {
                group.children.forEach(reflection => {
                    if (reflection.hasOwnDocument) {
                        addNavigationItem(reflection);
                    }
                });
            });
        }
        if (isModules) {
            project.groups[0].children.forEach(module => {
                const moduleNavigation = addNavigationItem(module);
                if (module.children) {
                    module.children.forEach(reflection => {
                        if (reflection.hasOwnDocument) {
                            addNavigationItem(reflection, moduleNavigation);
                        }
                    });
                }
            });
        }
        if (externalModulesNavigation.children.length) {
            navigation.children.push(externalModulesNavigation);
        }
        if (modulesNavigation.children.length) {
            navigation.children.push(modulesNavigation);
        }
        if (classesNavigation.children.length) {
            navigation.children.push(classesNavigation);
        }
        if (enumsNavigation.children.length) {
            navigation.children.push(enumsNavigation);
        }
        if (interfacesNavigation.children.length) {
            navigation.children.push(interfacesNavigation);
        }
        return navigation;
    }
    onPageEnd(page) {
        page.contents = page.contents ? MarkdownTheme.formatContents(page.contents) : '';
    }
    static formatContents(contents) {
        return contents
            .replace(/[\r\n]{3,}/g, '\n\n')
            .replace(/!spaces/g, '')
            .replace(/^\s+|\s+$/g, '');
    }
}
exports.default = MarkdownTheme;
